/*
*
*  CategoryAction.java
*
*  SPECT: Simple Portal-Enhancing Configuration Tool
*  Copyright (C) 1999, 2000 Ray Ontko & Co.
*  http://www.ontko.com/
*
*  This program is a simple-to-use configuration tool for Brio.Portal
*  (formerly ReportMart).
*
*  "Brio.Portal" is a trademark of Brio Technology, http://www.brio.com/ .
* 
* 
*  This program is free software; you can redistribute it and/or modify 
*  it under the terms of the GNU General Public License as published by 
*  the Free Software Foundation; either version 2 of the License, or 
*  (at your option) any later version. 
* 
*  This program is distributed in the hope that it will be useful, but 
*  WITHOUT ANY WARRANTY; without even the implied warranty of 
*  MERCHANTABILITY or FITNESS FOR A PARTICULAR PURPOSE. See the GNU General 
*  Public License for more details. 
* 
*  A copy of the GPL is available online the GNU web site: 
* 
*    http://www.gnu.org/copyleft/gpl.html
* 
*/ 

import java.io.*;
import java.util.*;
import com.sqribe.rm.*;

/******************************************************************
**
** CategoryAction parses and executes category-related commands from 
** the control file.  The allowed actions are create, update, and delete.
**
*******************************************************************/
public class CategoryAction extends BaseAction {

  private Repository repository;
  private static RMHashtable default_keywords;
  private RMHashtable keywords;
  private StreamTokenizer st;
  private int action;
  private int lineno;
  
  /**
  *
  * The static initializer sets the names and default values of allowed
  * keywords.
  *
  **/
  static {
    default_keywords = new RMHashtable();
    RMProperty rp = new RMProperty("NAME","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("DESCRIPTION","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("OWNER",RMLoader.username);
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("GROUP",RMLoader.group);
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("PERMISSIONS","740");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("EXPIRATION","");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("BROWSABLE","true");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("AUTODELETE","false");
    default_keywords.put(rp.name, rp);
    rp = new RMProperty("NEW-NAME","");
    default_keywords.put(rp.name, rp);
  }

  /**
  *
  * Constructor that initializes the keyword hashtable, and calls the
  * parse() and doAction() methods
  *
  **/
  public CategoryAction(Repository repository, int action, StreamTokenizer st, 
         int lineno) throws IOException{
    super(action, st, lineno);
    this.repository = repository;
    this.action = action;
    this.type = "CATEGORY";
    keywords = (RMHashtable)default_keywords.clone();
    parse(default_keywords, keywords);
    doAction();
  }

  /**
  *
  * doAction() determines which action is to be done and calls the 
  * appropriate method.
  *
  **/
  public int doAction() {
    switch (action) {
    case RMLoader.CREATE :
      createCategory();
      break;
    case RMLoader.UPDATE :
      updateCategory();
      break;
    case RMLoader.DELETE :
      deleteCategory();
      break;
    default :
      error("invalid action for CATEGORY, " + RMLoader.decodeAction(action) +
        ", cannot execute");
      printBadRecord();
    }
    return 0;
  }

  /**
  *
  * createCategory() checks config options and procedes to add the category
  * to ReportMart, or update it if it already exists and the config option
  * says to update
  *
  **/
  private void createCategory() {
    Category category;
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;


    // *** if NEW-NAME option is given, print appropriate error message
    //     regarding ignored option....

    try {
      category = repository.getCategory(name);
      // category exists.  check config for what to do
      switch (RMLoader.config.when_preexists) {
      case RMConfig.ERROR :
        error("category already exists: \"" + name + "\"");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("category already exists: \"" + name + "\"");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      case RMConfig.UPDATE :
        updateCategory();
        return;
      }
    }
    catch (ReportMartException rme) {}
    String msg = "";
    try {
      category = repository.addCategory(name);
      rmp = (RMProperty)keywords.get("DESCRIPTION");
      msg = "unable to set description, ";
      category.setDescription(rmp.value);
      rmp = (RMProperty)keywords.get("OWNER");
      msg = "unable to set user \"" + rmp.value + "\" as owner, ";
      category.setOwner(repository.getUser(rmp.value));
      rmp = (RMProperty)keywords.get("GROUP");
      msg = "unable to set group \"" + rmp.value + "\" as default group, ";
      category.setGroup(repository.getGroup(rmp.value));
      rmp = (RMProperty)keywords.get("PERMISSIONS");
      msg = "unable to set permissions, ";
      category.setPermissions(convertUmask(rmp.value));
      rmp = (RMProperty)keywords.get("EXPIRATION");
      msg = "unable to set expiration date, ";
      if (rmp.value.length() > 0)
        category.setExpiration(decodeDate(rmp.value));
      rmp = (RMProperty)keywords.get("BROWSABLE");
      msg = "unable to set browsable, ";
      category.setBrowsable(decodeBoolean(rmp.value));
      rmp = (RMProperty)keywords.get("AUTODELETE");
      msg = "unable to set autodelete, ";
      category.setAutoDelete(decodeBoolean(rmp.value));
      msg = "";
      category.update();
      log("Category \"" + name + "\" successfully created");
      printGoodRecord();
    }  // try {
    catch (ReportMartException rme) {
      error(msg + rme.getMessage() + ", unable to create category \"" + name +
        "\"");
      printBadRecord();
    }
  } // public void createCategory()

  /**
  *
  * Retrieve the named category and update the specified properties
  *
  **/
  private void updateCategory(){
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    Category category = null;
    String name = rmp.value;
    String msg = "";
 
    // Check to see if category exists
    try {
      category = repository.getCategory(rmp.value);
    }
    catch(ReportMartException rme) {
      switch (RMLoader.config.when_missing) {
      case RMConfig.ERROR :
        error("category missing, unable to update");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("category missing, unable to update");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      }
    }
 
    // Now update fields
    try {
      rmp = (RMProperty)keywords.get("DESCRIPTION");
      if (!rmp.is_default) {
        msg = "unable to set description, ";
        category.setDescription(rmp.value);
      }
      rmp = (RMProperty)keywords.get("OWNER");
      if (!rmp.is_default) {
        msg = "unable to set user \"" + rmp.value + "\" as owner, ";
        category.setOwner(repository.getUser(rmp.value));
      }
      rmp = (RMProperty)keywords.get("GROUP");
      if (!rmp.is_default) {
        msg = "unable to set group \"" + rmp.value + "\" as default group, ";
        category.setGroup(repository.getGroup(rmp.value));
      }
      rmp = (RMProperty)keywords.get("PERMISSIONS");
      if (!rmp.is_default) {
        msg = "unable to set permissions, ";
        category.setPermissions(convertUmask(rmp.value));
      }
      rmp = (RMProperty)keywords.get("EXPIRATION");
      if (!rmp.is_default) {
        msg = "unable to set expiration date, ";
        category.setExpiration(decodeDate(rmp.value));
      }
      rmp = (RMProperty)keywords.get("BROWSABLE");
      if (!rmp.is_default) {
        msg = "unable to set browsable, ";
        category.setBrowsable(decodeBoolean(rmp.value));
      }
      rmp = (RMProperty)keywords.get("AUTODELETE");
      if (!rmp.is_default) {
        msg = "unable to set autodelete, ";
        category.setAutoDelete(decodeBoolean(rmp.value));
      }
      rmp = (RMProperty)keywords.get("NEW-NAME");
      if (!rmp.is_default) {
        msg = "unable to set new name to \"" + rmp.value + "\", ";
        category.setName(rmp.value);
      }
      msg = "error in category .update() call";
      category.update();
      log("Category \"" + name + "\" was successfully updated");
    }
    catch (ReportMartException rme) {
      error(msg + rme.getMessage() + ", unable to update category \"" + name +
        "\"");
    }
  } // updateCategory()

  /**
  *
  * Permenantly remove the given category from Report Mart
  *
  **/
  private void deleteCategory(){
    RMProperty rmp = (RMProperty)keywords.get("NAME");
    String name = rmp.value;
    try {
      repository.getObjectByPath(name);
    }
    catch (ReportMartException rme) {
      switch(RMLoader.config.when_missing) {
      case RMConfig.ERROR :
        error("category \"" + name + "\" does not exist, cannot delete");
        printBadRecord();
        return;
      case RMConfig.WARNING :
        warning("category \"" + name + "\" does not exist, cannot delete");
      case RMConfig.IGNORE :
        printGoodRecord();
        return;
      }
    }
    Enumeration e = keywords.elements();
    boolean error_status = false;
    while (e.hasMoreElements()) {
      rmp = (RMProperty)e.nextElement();
      if (!rmp.name.equals("NAME") && !rmp.is_default) {
        switch(RMLoader.config.when_ignored_keyword) {
        case RMConfig.ERROR :
          error_status = true;
          error("ignored keyword, " +rmp.name + ", category will not be " +
            "deleted");
          break;
        case RMConfig.WARNING :
          warning("ignored keyword, " + rmp.name);
        case RMConfig.IGNORE :
        }
      }
    }
    if (error_status) {
      printBadRecord();   
      return;
    }
    try {
      repository.removeObjectByPath(name);
      log("Category \"" + rmp.value + "\" was successfully deleted");
    }
    catch (ReportMartException rme) {
      error("Error trying to delete category \"" + rmp.value +
                      "\", " + rme.getMessage());
    }
  } // private deleteCategory()
} // public class CategoryAction
